// Roland goes Demining!
// Minesweeper clone for Amstrad CPC 464 / 6128
// Amstrad 6128 features high score saving/loading
//
// (c) 2023
//     Program: SagaDS - Contact on CPCWiki forum
//     Music  : @chasebethea (http://chasebethea.com)

// arkos2 directory contains the arkos tracker 2 implementation and asm setinterruptHandler 
// which provides a better handling of interruptions  

// Naming a_main.c to put the main at the main adress of the program
// Main initialisation and game loop 

#include <cpctelera.h>

#include "game.h"
#include "ui.h"
#include "grid.h"
#include "pointer.h"
#include "render.h"
#include "music.h"
#include "clock.h"
#include "hiscore.h"

void main(void) 
{
   u32 loops;
   u8 tSpace, tU, tM, tA, tReturn, tSpeed, tHS, rc, uiState; 
   i8 dx, dy;

   // uiState : 
   // 0 : Starting demineur display titlescreen (and go into 1)
   // 1 : Intro screen key input loop - Animation / game mode / toggle autostart / 
   //     start / ESC / Music running - Goto 2 when Start ( unplugged : Goto 6 if ESC)
   // 2 : playfield waiting for autostart if needed - Time paused No music (then Goto 3)
   // 3 : playfield running : timer / Keys loop for moving cursor, dig, flag / End game checks = goto 4 or 5
   // 4 : End of play occured (win or loose), waiting for R - reset to 2 or ESC - Goto 0
   // 5 : Highscore input... QAOP choose letters / Return = Save if 6128 then goto 4
   // 6 : Exit loop and exit game

   tSpace = tU = tM = tA = tReturn = tSpeed = tHS = 0;
   uiState = 0;
   loops = 0;

   initDemineur();

   // Loop forever (or until ESC is pressed)
   while (1)
   {
      cpct_scanKeyboard ();

      switch (uiState)
      {
         case 0: // Display Menu , start music            
            titleScreen();
            changeGame(gGame); // To have a correct display of menu
            uiState++;
            break;
         case 1: // Game choice / Use QMark / Use Autostart / Start
            loops ++; // For randomiser
            if ( cpct_isKeyPressed (Key_1) && gGame != 0)
               changeGame (0); 
            if ( cpct_isKeyPressed (Key_2) && gGame != 1)
               changeGame (1);
            if ( cpct_isKeyPressed (Key_3) && gGame != 2)
               changeGame (2);
            if ( cpct_isKeyPressed (Key_4) && gGame != 3)
               changeGame (3);

            if ( !tM && cpct_isKeyPressed (Key_M) )
            {
               tM = 1;
               nextMusicOption();
            }
            else if ( tM && !cpct_isKeyPressed (Key_M) )
              tM = 0;

            if ( !tU && cpct_isKeyPressed (Key_U) )
            {
               tU = 1;
               gUseQMark = 1 - gUseQMark;
               displayMenuValues();
            }
            else if ( tU && !cpct_isKeyPressed (Key_U) )
              tU = 0;

            if ( !tA && cpct_isKeyPressed (Key_A) )
            {
               tA = 1;
               gUseAutoStart = 1 - gUseAutoStart;
               displayMenuValues();
            }
            else if ( tA && !cpct_isKeyPressed (Key_A) )
              tA = 0;

            if ( cpct_isKeyPressed (Key_S) )
            {
               cpct_setSeed_mxor(loops); // Randomiser
               uiState++;
            }

            updateMenuAnimation();
            break;
         case 2:  // Display grid
            launchGame();
            uiState++;
            initBombExplEnd (); // For interactive ending
            break;
         case 3:  // Interactive play
            dx = 0; dy = 0;

            // Auto Keys
            if ( cpct_isKeyPressed (Key_Q) || cpct_isKeyPressed (Joy0_Up) ) 
               dy = -1;
            if ( cpct_isKeyPressed (Key_A) || cpct_isKeyPressed (Joy0_Down))
               dy = 1;
            if ( cpct_isKeyPressed (Key_O) || cpct_isKeyPressed (Joy0_Left)) 
               dx = -1;
            if ( cpct_isKeyPressed (Key_P) || cpct_isKeyPressed (Joy0_Right))
               dx = 1;
            if (dx != 0 || dy != 0)
               movePointer (dx,dy);
            
            // Not auto Keys: active when pressed waiting to unpress for next one 
            if (!tSpace && ( cpct_isKeyPressed (Key_Space) || cpct_isKeyPressed (Joy0_Fire1) ) )
            {
               tSpace = 1;
               flagCase ();
            }
            else if ( tSpace && !(cpct_isKeyPressed (Key_Space) || cpct_isKeyPressed (Joy0_Fire1)) )
               tSpace = 0;

            if (!tReturn && (cpct_isKeyPressed (Key_Return) || cpct_isKeyPressed (Joy0_Fire2) ) )
            {
               tReturn = 1;
               rc = digCase ();
               if (rc == 1)  // 1 : Kaoom, 2 : success
               {
                  displayFailure () ;
                  uiState = 4;
               }
               else if ( rc == 2 )
               {
                  uiState = 4;
                  if ( displaySuccess () == 1)
                  {
                     initHSInput(gGame);
                     uiState = 5;
                     loops = 0; // For better control of key speed
                     tHS = 1;
                  }
               }
            }
            else if ( tReturn && !(cpct_isKeyPressed (Key_Return) || cpct_isKeyPressed (Joy0_Fire2)) )
               tReturn = 0;

            if (!tSpeed && cpct_isKeyPressed (Key_S))
            {
               togglePointerSpeed ();
               tSpeed = 1;
            }
            else if ( tSpeed && !cpct_isKeyPressed (Key_S) )
               tSpeed = 0;

            if ( cpct_isKeyPressed (Key_R) ) 
               launchGame();

            if ( cpct_isKeyPressed (Key_Esc) ) 
            {
               uiState = 4;
               pauseGameTimer ();
            }

            break;
         case 4:
            updateBombExpl ();
            if ( cpct_isKeyPressed (Key_R) )
               uiState = 2;

            if ( cpct_isKeyPressed (Key_Esc) ) 
            {
               stopMusic();
               clearScreenAnimation ();
               uiState = 0;
            }
            break;
         case 5:
            // requesting a trigram for high score

            // Small smoothing of key speed 
            loops ++;
            if (loops < 10)
               break;
            else
               loops = 0;

            if ( cpct_isKeyPressed (Key_Q) || cpct_isKeyPressed (Joy0_Up)) 
               prevHSChar();
            if ( cpct_isKeyPressed (Key_A) || cpct_isKeyPressed (Joy0_Down))
               nextHSChar();
            if ( cpct_isKeyPressed (Key_O) || cpct_isKeyPressed (Joy0_Left)) 
               prevHSPos();
            if ( cpct_isKeyPressed (Key_P) || cpct_isKeyPressed (Joy0_Right))
               nextHSPos();

            if ( !tHS && cpct_isKeyPressed (Key_Return) )
            {
               endHSDisplay();
               uiState = 4;
            }
            else if ( tHS && !cpct_isKeyPressed (Key_Return) )
               tHS = 0;

            break;
      }
  
      if (uiState == 6)
         break;
      cpct_waitVSYNC ();
   }
   exitDemineur();
}
