#include "game.h"

#include "disk.h"
#include "ui.h"
#include "render.h"
#include "pointer.h"
#include "clock.h"
#include "grid.h"
#include "music.h"
#include "sprites.h"
#include "hiscore.h"
#include "asmroutines.h"

// Game Globals

u8 gGame; // game number (from 0 to 3)

u8  gUseQMark; // Game options
u8  gUseAutoStart; // 

u8  gGridInitialized; // grid play variables
u16 gNbFlag; // Nb of played flags
u16 gNbDig; // Nb of successful digs

u8 *pvmem; // a handy screen pointer 

u16 firmAdress; // Local storage of firmware

void initDemineur ()
{
   initHiScore(); // Inits Tape Hiscore
   initDisk(); // While firmware is still there, loads HiScore file if disk, protects saaves if Tape

   disableFirmware (); // And initTimer

   initUI();
   initRender ();
   initPointer ();
   initMusic();
   initSprites();

   gGame = 1; // Default game for titleScreen (intermediate)
   gUseQMark = 0;
   gUseAutoStart = 1;
}

void restoreFirmware ()
{
    cpct_reenableFirmware (firmAdress);
}

void disableFirmware ()
{
   firmAdress = cpct_disableFirmware ();
   initTimer(); // To set new interrupt handler
}

void exitDemineur ()
{
    restoreFirmware ();
}

void changeGame (i8 game)
{
   gGame = game;
   initGrid();
   displayMenuValues();
}

void launchGame ()
{
   stopMusic(); // Stop menu song if any
   pauseGameTimer ();
   gGridInitialized = 0;
   clearScreenAnimation ();
   initDig();
   setGridOffset ();
   printGrid ();
   displayPlayscreen ();
   displayGridInfos();
   initPointerSpeed ();
   savePointerScreen ();
   printPointerScreen ();
   resetGameTimer ();
   displayClock(0);
   startMusic (MUS_THEM1);
}

void getStringFromWord(char *Buff, u16 val)
{
    // Writes words backwards from char #4 to #0 (0 to 65536) - from 4 to 2 max in this game
    // Needs a buffer with 5 pos mini
    u16 tmpVal;
    u8  posChar;

    // init buffer with spaces and a 0 at the end
    Buff[0] = ' ';
    Buff[1] = ' ';
    Buff[2] = ' ';
    Buff[3] = ' ';
    Buff[4] = ' ';
    Buff[5] = 0;

    posChar = 4;
    do
    {
        tmpVal = val / 10;
        Buff[posChar--] = val - 10 * tmpVal + '0';
        val = tmpVal;
    } while (tmpVal > 0);
}
