#include "grid.h"

#include "game.h"
#include "render.h"
#include "pointer.h"
#include "clock.h"
#include "ui.h"
#include "music.h"

// Sizes and nbBombs of available games
const u8 _gridSizeX [4] = { 8, 16, 30, 30  }; 
const u8 _gridSizeY [4] = { 8, 16, 16, 24  }; 
const u8 _nbBomb    [4] = {10, 40, 99, 200 };

// Playfields
u8 bombTable [32*24]; // 0 : no bomb, 1-8 : nbBombAround, 128 bit 7 set : bomb
u8 digTable  [32*24]; // 0 : not used, 1 : digged, 2 : flagged, 3 : ?

// Current game values
i8  gGridWidth, gGridHeight; // signed for easy navigation and 0 handling
u8  gGridNbBomb;
u16 gGridSize;

void initGrid ()
{
   // Using gGame to set grid dimensions
   gGridWidth  = _gridSizeX[gGame];
   gGridHeight = _gridSizeY[gGame];
   gGridNbBomb = _nbBomb[gGame];
   gGridSize   = (u16) (gGridWidth * gGridHeight);
}

void initDig ()
{
   // Reset dig values and scoring infos
   cpct_memset (digTable,0,gGridSize);

   gNbFlag = 0;
   gGridInitialized = 0;
   gNbDig = 0;
   gShowBomb = 0;
}

void initBombField (i8 ptX , i8 ptY )
{
   u16 offset,offset2;
   i8 x,y;
   u8 bomb,rand;

   cpct_memset (bombTable,0,gGridSize);

   // Works well for population < 50%
   for  (bomb=0 ; bomb < gGridNbBomb ; bomb++)
   { 
      do // Look for case without a bomb (works better with nbbomb < gridSizeX * gridSizeY / 2)
      { 
         i8 xTmp, yTmp;
         do  // To keep a 0 in ptX, ptY when autostart
         {
            rand = cpct_getRandom_mxor_u8();
            while (rand >= (u8)gGridWidth)  rand -= (u8)gGridWidth;
            x = (i8) rand;
            rand = cpct_getRandom_mxor_u8();
            while (rand >= (u8)gGridHeight)  rand -= (u8)gGridHeight;
            y = (i8) rand;
            xTmp = x-ptX;
            yTmp = y-ptY;
         } while (  gUseAutoStart && xTmp >= -1 && xTmp <= 1 && yTmp >= -1 && yTmp <= 1 );
         offset = y * gGridWidth + x;
      } while ( bombTable[offset] & 128 );
      
      bombTable[offset] += 128; // Set a bomb (don't care of previous value, now it's a bomb)

      // Increase nb of bombs inside adjacent cases (don't care if a bomb exist, it will be 129,130,...)
      // Check previous line 
      if ( y > 0 ) // if not the first
      {
         offset2 = offset - gGridWidth - 1;
         if ( x > 0 )   bombTable[offset2] += 1;
         bombTable[++offset2] += 1;
         if (x < gGridWidth - 1) bombTable[++offset2] += 1;
      }

      // Same line
      if ( x > 0 )   bombTable[offset  - 1] += 1;
      if (x < gGridWidth - 1) bombTable[offset + 1] += 1;

      // Next line
      if ( y < gGridHeight - 1) // If not last
      {
         offset2 = offset + gGridWidth - 1;
         if ( x > 0 )   bombTable[offset2] += 1;
         bombTable[++offset2] += 1;
         if (x < gGridWidth - 1) bombTable[++offset2] += 1;
      }
   }
   gGridInitialized = 1;
}

// checkFlag is used when we autodig around a number and not from a 0 propagation
// Also a 0 propagation can clear 'bad flags'
u8 autoDig (u8 gridX, u8 gridY, u16 offset, u8 checkFlag )
{
   i16 localOffset;
   u8 *pDig;
   i8 localGridX, localGridY, dx, dy;
   u8 rc = 0;

   localOffset = offset - gGridWidth - 1;
   for (dy=-1;dy<=1;dy++)
   {
      for (dx=-1;dx<=1;dx++)
      {
         localGridX = gridX + dx;
         localGridY = gridY + dy;
         if ( localGridX >=0 && localGridX < gGridWidth
           && localGridY >=0 && localGridY < gGridHeight )
         {
            pDig = &digTable [localOffset];
            if ( *pDig != 1 &&  ( *pDig !=2 || !checkFlag ) ) 
            {
               if ( bombTable[localOffset] & 128)
                  rc = 1; // Kaboom if there was a boom here (bad flag around number case)
               else if ( *pDig == 2 ) // Bad flag removal (0 propagation)
                  gNbFlag--;
               *pDig = 1;
               gNbDig++;
               pvmem = getGridAdress (localGridX, localGridY) ;
               printCell (localOffset, 1, pvmem); // Show the dig
               if ( ! bombTable [localOffset] ) // If still a 0 continue
               {
                  if (autoDig (localGridX,localGridY,localOffset, checkFlag))
                     rc = 1; // Propagate any kaboom if needed
               }
            }
         }
         localOffset ++;
      }
      localOffset += gGridWidth - 3;
   }
   return rc;
}

void flagCase ()
{
   u16 gridOffset;
   u8  *pDigVal, newVal;
   u8  gridX, gridY;

   // Outside pointing area
   if ( !getGridFromPointer ( &gridX, &gridY)) return;

   gridOffset = gridX + gridY * gGridWidth;
   pDigVal = &digTable [gridOffset];
   newVal = *pDigVal;

   // Reprint sub grid under pointer
   clearPointerScreen ();

   switch (newVal)
   {
      case 0: // Not digged -> Flagged
         gNbFlag++;
         displayGridInfos();
         newVal = 2; 
         playSFX(SFX_FLAG);
         break;
      case 1: // Digged -> nothing
         playSFX(SFX_DENY);
         break;
      case 2: // Flagged -> either ? or Reset based on gUseQMark
         gNbFlag--;
         displayGridInfos();
         newVal = (gUseQMark?3:0);
         playSFX(SFX_DFLG);
         break;
      case 3: // ? -> Reset
         newVal = 0; 
         break;
   }
   *pDigVal = newVal;

   pvmem = getGridAdress ( gridX, gridY);
   printCell (gridOffset, newVal, pvmem);

   // reprint pointer
   savePointerScreen ();
   printPointerScreen ();
}

// Check if falgs around are a good match for the grid value, if yes autodig
u8 checkAround ( i8 gridX, i8 gridY, u16 gridOffset)
{
   u8 fieldVal, nbFlag, rc;
   i16 localOffset;
   i8 localGridX, localGridY, dx, dy;

   rc = 0;

   fieldVal = bombTable [gridOffset];
   if (fieldVal != 0) // If 0 autodig is already done
   {
      nbFlag = 0;
      localOffset = gridOffset - gGridWidth - 1;
      for (dy=-1;dy<=1;dy++)
      {
         for (dx=-1;dx<=1;dx++)
         {
            localGridX = gridX + dx;
            localGridY = gridY + dy;
            if ( localGridX >=0 && localGridX < gGridWidth
              && localGridY >=0 && localGridY < gGridHeight )
            {
               if (digTable [localOffset] == 2)
                  nbFlag ++;
            }
            localOffset ++;
         }
         localOffset += gGridWidth - 3;
      }

      if ( nbFlag == fieldVal )
      {
         playSFX (SFX_DIG);
         rc = autoDig (gridX,gridY,gridOffset,1);
         displayGridInfos();
      }
      else
         playSFX(SFX_DENY); // Too much or to few flags around
   }
   return rc;
}


u8 digCase ()
{
   u16 gridOffset;
   u8  *pDigVal;
   u8  fieldVal, rc, gridX, gridY;

   // Outside pointing area ?
   if ( !getGridFromPointer ( &gridX, &gridY)) return 0;

   // Initialize the grid if needed (using pointer coordinates if autostart is used)
   if (!gGridInitialized) initBombField (gridX, gridY);

   pauseGameTimer (); // Let's pause the timer during the process

   clearPointerScreen (); // Reprint sub grid under pointer

   gridOffset = gridX + gridY * gGridWidth;
   pDigVal = &digTable [gridOffset];
   rc = 0;
   if ( *pDigVal == 0 || *pDigVal == 3) // Not digged or ? - let's dig
   {
      *pDigVal = 1;
      gNbDig++;
      pvmem = getGridAdress ( gridX, gridY);
      printCell (gridOffset, 1, pvmem);
      fieldVal = bombTable [gridOffset];
      if ( fieldVal & 128  ) // Kaboom - RIP
      {
         playSFX(SFX_BOOM);
         rc = 1;
      }
      else
      {
         playSFX (SFX_DIG);
         if ( ! fieldVal ) // Zero bomb: autodig without flag checks
            autoDig (gridX, gridY, gridOffset, 0);
      }
      displayGridInfos();
   }
   else if ( *pDigVal == 1 ) // Digging a digged case - Check flags
   {
      // Check if already digged value is surrounded by exact number of flags, if yes, autoDig around it
      // Can potential goes boom
      rc = checkAround (gridX,gridY,gridOffset);
   }

   if (!rc) // No Kaboom ? If not checked, SUCCESS can happen if last case is kaboom
   {
      if (gNbDig == gGridSize-gGridNbBomb)
         rc = 2; // SUCCESS
      else // Continue game
      {
         unpauseGameTimer();
         savePointerScreen ();
         printPointerScreen ();
      }
   }

   return rc;
}
